package com.mortals.xhx.base.framework.aspect;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.net.NetUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.common.pdu.access.AccessLogPdu;
//import com.mortals.xhx.system.MessageProducer;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Profile;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.util.ContentCachingRequestWrapper;

import javax.servlet.http.HttpServletRequest;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.Map;

/**
 * 打印每个请求的入参、出参等信息
 *
 * @author: zxfei
 * @date: 2022/4/20 9:24
 */
//@Aspect
//@Component
@Slf4j
//@Order(1)
@Profile({"default", "develop", "test"})
public class WebLogAspect {

    @Value("${spring.application.name:base-platform}")
    private String appName;
//    @Autowired
//    private MessageProducer messageProducer;

    private static final String TRACE_ID = "traceId";
    private static final String SPAN_ID = "spanId";
    private static final String PSPAN_ID = "pspanId";

    @Pointcut("execution(public * com.mortals..*Controller.*(..))")
    public void webLog() {
    }


    @Pointcut("execution(public * com.mortals.xhx.base.framework.exception.ExceptionHandle.*(..))")
    public void exceptions() {
    }

    /**
     * 只在进入controller时记录请求信息
     */
    @Before("webLog()")
    public void doBefore(JoinPoint joinPoint) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        // log.debug("请求路径 {} ,进入方法 {}", request.getRequestURI(), joinPoint.getSignature().getDeclaringTypeName() + ":" + joinPoint.getSignature().getName());
        MDC.put("req", getRequestInfo(request).toJSONString());
        MDC.put("reqParams", getRequestInfoParams(request).toJSONString());
        MDC.put("startTime", String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 打印请求日志,包含异常
     */
    @AfterReturning(pointcut = "webLog()|| exceptions()", returning = "result")
    public void afterReturning(Object result) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        String traceId = request.getHeader(TRACE_ID);
        String pspanId = request.getHeader(PSPAN_ID);
        String spanId = request.getHeader(SPAN_ID);
        Map<String, String> map = MDC.getCopyOfContextMap();
        if (map != null && result != null) {
            String startTime = map.getOrDefault("startTime", String.valueOf(System.currentTimeMillis()));
            long takeTime = (System.currentTimeMillis() - Long.parseLong(startTime));

            if (result instanceof String) {
                log.debug(" \n 请求路径:{} 耗时：{}ms \n traceId：{} pspanId：{} spanId：{}  \n 请求报文：{} \n 响应报文：{}"
                        , request.getRequestURI(), takeTime, traceId, pspanId, spanId, map.getOrDefault("req", ""), result);
            } else {

                log.debug(" \n 请求路径:{} 耗时：{}ms \n traceId：{} pspanId：{} spanId：{} \n 请求报文：{} \n 响应报文：{}"
                        , request.getRequestURI(), takeTime, traceId, pspanId, spanId, map.getOrDefault("req", ""), JSON.toJSONString(result));
            }

            if (!ObjectUtils.isEmpty(pspanId) && !ObjectUtils.isEmpty(spanId)) {
                //paspId
                spanId = String.valueOf(Integer.parseInt(spanId) + 1);
                pspanId = spanId;
            }else{
                spanId="1";
                pspanId="0";
            }

            String ua = request.getHeader("User-Agent");
            AccessLogPdu accessLogPdu = new AccessLogPdu();
            accessLogPdu.initAttrValue();
            accessLogPdu.setAppName(appName);
            accessLogPdu.setTraceID(traceId == null ? IdUtil.objectId() : traceId);
            accessLogPdu.setPspanId(Integer.parseInt(pspanId));
            accessLogPdu.setSpanId(Integer.parseInt(spanId));
            accessLogPdu.setSchemaData(request.getScheme());
            accessLogPdu.setHostName(NetUtil.getLocalHostName());
            accessLogPdu.setUri(request.getRequestURI());
            accessLogPdu.setTargetServer(appName);
            accessLogPdu.setRequestIp(request.getRemoteAddr());
            accessLogPdu.setUa(ua==null?"":ua);
            accessLogPdu.setRequestTime(DateUtils.getDate(Long.parseLong(startTime)));
            accessLogPdu.setLogTime(new Date());
            accessLogPdu.setMethod(request.getMethod());
            accessLogPdu.setResponseTime(new Date());
            accessLogPdu.setDuration(takeTime);

            accessLogPdu.setRequestData(map.getOrDefault("reqParams", ""));
            if (result instanceof String) {
                accessLogPdu.setResponseData(result.toString());
            } else {
                accessLogPdu.setResponseData(JSON.toJSONString(result));
            }
            log.info("accessLog:{}",JSON.toJSONString(accessLogPdu));
            //messageProducer.syncAccessSend(accessLogPdu);

        }
    }

    /**
     * 读取请求信息,如果是表单则转换为json
     */
    private JSONObject getRequestInfo(HttpServletRequest req) {
        JSONObject requestInfo = new JSONObject();
        try {
            StringBuffer requestURL = req.getRequestURL();
            requestInfo.put("requestURL", requestURL);
            String method = req.getMethod();
            requestInfo.put("method", method);
            if (req.getQueryString() != null) {
                requestInfo.put("queryString", URLDecoder.decode(req.getQueryString(), "UTF-8"));
            }
            String remoteAddr = req.getRemoteAddr();
            requestInfo.put("remoteAddr", remoteAddr);
            if (req instanceof ContentCachingRequestWrapper) {
                ContentCachingRequestWrapper wrapper = (ContentCachingRequestWrapper) req;
                String bodyStr = new String(wrapper.getContentAsByteArray(), StandardCharsets.UTF_8);
                if (bodyStr.startsWith("{")) {
                    JSONObject jsonObject = JSON.parseObject(bodyStr);
                    requestInfo.put("requestBody", jsonObject);
                }
            }
        } catch (Exception e) {
            log.error("解析请求失败", e);
            requestInfo.put("parseError", e.getMessage());
        }
        return requestInfo;
    }


    /**
     * 读取请求信息,如果是表单则转换为json
     */
    private JSONObject getRequestInfoParams(HttpServletRequest req) {
        JSONObject requestInfo = new JSONObject();
        try {

            if (req.getQueryString() != null) {
                requestInfo.put("queryString", URLDecoder.decode(req.getQueryString(), "UTF-8"));
            }
            if (req instanceof ContentCachingRequestWrapper) {
                ContentCachingRequestWrapper wrapper = (ContentCachingRequestWrapper) req;
                String bodyStr = new String(wrapper.getContentAsByteArray(), StandardCharsets.UTF_8);
                if (bodyStr.startsWith("{")) {
                    JSONObject jsonObject = JSON.parseObject(bodyStr);
                    requestInfo.put("requestBody", jsonObject);
                }
            }
        } catch (Exception e) {
            log.error("解析请求失败", e);
            requestInfo.put("parseError", e.getMessage());
        }
        return requestInfo;
    }
}
