package com.mortals.xhx.daemon.task;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SiteMatterPdu;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.feign.site.ISiteMatterFeign;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import com.mortals.xhx.module.site.service.SiteMatterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 同步事项列表
 */
@Slf4j
@Service("SyncSiteMatterTask")
public class SyncSiteMatterTaskImpl implements ITaskExcuteService {

    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private ISiteMatterFeign siteMatterFeign;
    @Autowired
    private SiteMatterService siteMatterService;

    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("开始同步事项列表！");
        SitePdu sitePdu = new SitePdu();
        sitePdu.setId(1L);
        Rest<List<SitePdu>> siteRest = siteFeign.getFlatSitesBySiteId(sitePdu);
        if (siteRest.getCode() == YesNoEnum.YES.getValue()) {
            log.info("总数量：{}",siteRest.getData().size());
            siteRest.getData().forEach(site -> {
                siteMatterService.getDao().delete(new SiteMatterQuery().siteId(site.getId()));
                int page=100;
                int pageNum=100;
                for(int i=1;i<=page;i++){
                    SiteMatterPdu siteMatterPdu = new SiteMatterPdu();
                    siteMatterPdu.setPage(i);
                    siteMatterPdu.setSize(pageNum);
                    siteMatterPdu.setSiteId(site.getId());
                    log.info("请求列表:"+i);
                    Rest<RespData<List<SiteMatterPdu>>> siteMatterRest = siteMatterFeign.list(siteMatterPdu);
                    if (siteMatterRest.getCode() == YesNoEnum.YES.getValue()) {
                        if(siteMatterRest.getData().getData().size()==0){
                            log.info("数据没有！跳出循环");
                            break;
                        }
                        //删除后新增
                        log.info("事项总数量：{}",siteMatterRest.getData().getData().size());
                        List<SiteMatterEntity> siteMatterEntities = siteMatterRest.getData().getData().stream().map(siteMatter -> {
                            SiteMatterEntity sheetMatterEntity = new SiteMatterEntity();
                            sheetMatterEntity.initAttrValue();
                            sheetMatterEntity.setId(siteMatter.getMatterId());
                            sheetMatterEntity.setSiteId(siteMatter.getSiteId());
                            sheetMatterEntity.setMatterName(siteMatter.getMatterName());
                            sheetMatterEntity.setMatterCode(siteMatter.getMatterCode());
                            sheetMatterEntity.setDeptId(siteMatter.getDeptId());
                            sheetMatterEntity.setDeptCode(siteMatter.getDeptCode());
                            sheetMatterEntity.setDeptName(siteMatter.getDeptName());
                            sheetMatterEntity.setAreaCode(siteMatter.getAreaCode());
                            sheetMatterEntity.setSource(siteMatter.getSource());
                            sheetMatterEntity.setEventTypeShow(siteMatter.getEventTypeShow());
                            sheetMatterEntity.setViewsCount(0l);
                            sheetMatterEntity.setIsConvert(0);
                            return sheetMatterEntity;
                        }).collect(Collectors.toList());

                        if (!ObjectUtils.isEmpty(siteMatterEntities)) {
                            // sheetMatterService.getDao().delete(new SheetMatterQuery().siteId(site.getId()));
                            log.info("新增数量：{}",siteMatterEntities.size());
                            siteMatterService.save(siteMatterEntities);
                  /*         List<List<SheetMatterEntity>> partition = ListUtil.partition(sheetMatterEntities, 500);
                           for (List<SheetMatterEntity> matterEntities : partition) {
                           }*/

                        }
                    }else{
                        log.info("请求错误，code:{}",siteMatterRest.getCode());
                    }
                }

            });
        }
    }
    @Override
    public void stopTask(ITask task) throws AppException {

    }
}
