package com.mortals.xhx.module.site.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import com.mortals.xhx.module.site.service.SiteMatterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;

/**
*
* 站点事项
*
* @author zxfei
* @date 2023-03-10
*/
@RestController
@RequestMapping("site/matter")
public class SiteMatterController extends BaseCRUDJsonBodyMappingController<SiteMatterService,SiteMatterEntity,Long> {

    @Autowired
    private ParamService paramService;

    public SiteMatterController(){
        super.setModuleDesc( "站点事项");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "source", paramService.getParamBySecondOrganize("SiteMatter","source"));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(SiteMatterEntity query, Map<String, Object> model, Context context) throws AppException {
        Map<String,String> orderMap = new HashMap<>();
        orderMap.put("viewsCount","DESC");
        query.setOrderCols(orderMap);
        super.doListBefore(query,model,context);
    }

    @Override
    protected int viewAfter(Long id, Map<String, Object> model, SiteMatterEntity entity, Context context) throws AppException {
        this.service.addViewsCount(id);
        return 1;
    }

    /**
     * 添加事项到单事项
     */
    @PostMapping(value = "addToSingle")
    public String addToSingle(@RequestBody Map<String, Object> map) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<String, Object>();
        String siteMatterIds = (String) map.get("siteMatterIds");
        try {
            Rest<String> rest = this.service.addToSingle(siteMatterIds, getContext());
            jsonObject.put(KEY_RESULT_MSG, rest.getMsg());
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    @PostMapping(value = "sublist")
    @UnAuth
    public Rest<Object> sublist(@RequestBody SiteMatterEntity query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询差集" + this.getModuleDesc();
        int code=VALUE_RESULT_SUCCESS;
        try {
            PageInfo pageInfo = buildPageInfo(query);
            if(ObjectUtils.isEmpty(query.getEventTypeShowNotList())){
                ArrayList<String> notList = new ArrayList<>();
                notList.add("行政处罚");
                query.setEventTypeShowNotList(notList);
            }
            Result<SiteMatterEntity> result = this.getService().findSubList(query, pageInfo, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            parsePageInfo(model, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }
}